body{font-family:Open Sans,Helvetica,sans-serif;font-size:14px;padding:0;margin:0}.center{max-width:800px;margin:0 auto}header{font-size:20px;font-weight:700;padding:24px}header a{color:#000;display:block;text-decoration:none}h1{background-color:#2075b1;color:#fff;font-size:16px;padding:16px;margin:0}summary{padding:10px 20px;background-color:#24425c;color:#fff;margin-bottom:10px}#status{text-align:center}#status>div{display:inline-block;min-width:90px;height:75px;margin:5px;padding:20px;vertical-align:middle}#status>div>div{padding:10px}#stats span{display:inline-block;min-width:30px}#status .active,#status .inactive{font-size:x-large;border-top:1px solid grey}#status .inactive{color:grey}main{padding:20px}table{width:100%;border-spacing:0;border-collapse:collapse}th{font-weight:700;background-color:#eee}td,th{text-align:left;padding:10px;border:1px solid silver;border-radius:4px;margin:0}tr.extendlist a{display:block;text-decoration:none}tr.extendlist td{padding:5px 10px;text-align:center}#preview img{max-width:90%;max-height:300px}td.more{cursor:pointer;font-style:italic}td.more:hover{background-color:#eee}
/*# sourceMappingURL=/static/style.aefeac67.css.map */